/*******************************************************************
 *
 *	File:		CArloConfig.cpp
 *
 *	Author:		Peter van Sebille (peter@yipton.net)
 *
 *	(c) Copyright 2001, Peter van Sebille
 *	All Rights Reserved
 *
 *******************************************************************/

#include <e32std.h>
#include <f32file.h>
#include <e32svr.h>
#include <e32cons.h>

#include "CArloConfig.h"
#include "version.h"

#define KKernelString		_L("image")
#define KInitRdString		_L("initrd")
#define KBootString			_L("boot")
#define KForegroundColour	_L("fgcolour")
#define KBackgroundColour	_L("bgcolour")
#define KArchNumber			_L("arch")

#define KTimeoutString		_L("timeout")
#define KDefaultBoot		_L("default")
#define KMachineName		_L("machine")
#define KUseBoot2nd			_L("useboot2nd")



CArloConfig::~CArloConfig()
{
	delete iArloBootItems;
	iArloBootItems = NULL;
}

CArloConfig::CArloConfig()
{
}


CArloConfig* CArloConfig::ReadConfigFileLC()
{
	CArloConfig*	arloConfig = new(ELeave) CArloConfig;
	CleanupStack::PushL(arloConfig);
	arloConfig->ConstructL();
	return arloConfig;
}

void CArloConfig::NewParsedBootItemL(const TDesC& aBootItem)
{
	TArloBootItem	item;

	if (iArloBootItems->Count() < 10)
	{
		item.iKernelFileName = _L("");
		item.iInitrdFileName = _L("");
		item.iKernelLoadAdr = 0;
		item.iInitrdLoadAdr = 0;

		if (aBootItem.Length() > KMaxConfigTextString)
			item.iDescription = aBootItem.Left(KMaxConfigTextString);
		else
			item.iDescription = aBootItem;
		item.iBootString = _L("");
		item.iForegroundColour = 0;
		item.iBackgroundColour = 0;
		item.iHasColoursSet = EFalse;
		item.iArchNumber = 0;

		iArloBootItems->AppendL(item);
	}
}

TBool CArloConfig::GetNumber(TLex& aLex, TUint& aNumber)
{
	TChar		c1, c2;
	TBool		success = EFalse;

	if (aLex.Remainder().Length() > 2)
	{
		c1= aLex.Get();
		c2= aLex.Get();
		if (c1 == '0' && (c2 == 'x' || c2 == 'X'))
		{
			success = aLex.Val(aNumber, EHex) == KErrNone;
		}
		else
		{
			aLex.UnGet();
			aLex.UnGet();
		}
	}

	if (!success)
	{
		success = aLex.Val(aNumber, EDecimal) == KErrNone;
	}
	return success;
}


void CArloConfig::ParseFileNameAndLoadAdr(const TDesC& aString, TDes& aFileName, TUint& aLoadAdr)
{
	TInt	pos = aString.LocateF(TChar('@'));

	if ((pos > 0) && (pos + 1 < aString.Length()))
	{
		TLex		input(aString.Left(pos));

		input.SkipSpaceAndMark();
		input.SkipCharacters();
		if (input.TokenLength() != 0)
		{
			TPtrC		token = input.MarkedToken();

			input.Assign(aString.Mid(pos+1));
			input.SkipSpaceAndMark();
			if (!GetNumber(input, aLoadAdr))
				aLoadAdr = 0;
	
			if (token.Length() > aFileName.MaxLength())
				aFileName.Copy(token.Left(aFileName.MaxLength()));
			else
				aFileName.Copy(token);
		}
	}
	else
	{
		if (aString.Length() > aFileName.MaxLength())
			aFileName.Copy(aString.Left(aFileName.MaxLength()));
		else
			aFileName.Copy(aString);
		aLoadAdr = 0;
	}
}

void CArloConfig::ParseItems(const TDesC& aToken, const TDesC& aValue)
{
//	RDebug::Print(_L("  token = -%S-"), &aToken);
//	RDebug::Print(_L("  value = -%S-"), &aValue);
	
	TInt	count = iArloBootItems->Count();
	if (aToken.CompareF(KKernelString) == 0)
	{
		if (count > 0 && count<10)
		{
			TUint	loadAdr;
			
			ParseFileNameAndLoadAdr(aValue, iArloBootItems->At(count - 1).iKernelFileName, loadAdr);
			iArloBootItems->At(count - 1).iKernelLoadAdr = loadAdr;
		}
	}
	else if (aToken.CompareF(KInitRdString) == 0)
	{
		if (count > 0 && count<10)
		{
			TUint	loadAdr;
	
			ParseFileNameAndLoadAdr(aValue, iArloBootItems->At(count - 1).iInitrdFileName, loadAdr);
			iArloBootItems->At(count - 1).iInitrdLoadAdr = loadAdr;
		}
	}
	else if (aToken.CompareF(KBootString) == 0)
	{
		if (count > 0 && count<10)
			iArloBootItems->At(count - 1).iBootString = aValue;
	}
	else if (aToken.CompareF(KTimeoutString) == 0)
	{
		TLex	input(aValue);

		TInt	timeout;
		if (input.Val(timeout) == KErrNone)
			iTimeout = timeout;
	}
	else if (aToken.CompareF(KDefaultBoot) == 0)
	{
		iDefaultBootItemString = aValue;
	}
	else if (aToken.CompareF(KMachineName) == 0)
	{
		iMachineName = aValue;
		iHasMachineName = ETrue;
	}
	else if (aToken.CompareF(KForegroundColour) == 0)
	{
		TLex	input(aValue);

		TUint	colour;
		if (GetNumber(input, colour))
		{
			iArloBootItems->At(count - 1).iForegroundColour = colour;
			iArloBootItems->At(count - 1).iHasColoursSet = ETrue;
		}
	}
	else if (aToken.CompareF(KBackgroundColour) == 0)
	{
		TLex	input(aValue);

		TUint	colour;
		if (GetNumber(input, colour))
		{
			iArloBootItems->At(count - 1).iBackgroundColour = colour;
			iArloBootItems->At(count - 1).iHasColoursSet = ETrue;
		}
	}
	else if (aToken.CompareF(KArchNumber) == 0)
	{
		TLex	lex(aValue);

		TUint	archno;
		if (CArloConfig::GetNumber(lex, archno))
			iArloBootItems->At(count - 1).iArchNumber = (TInt) archno;
	}
	else if (aToken.CompareF(KUseBoot2nd) == 0)
	{
		iHasUseBoot2nd = ETrue;
		if (((aValue.CompareF(_L("yes")) == 0) || (aValue.CompareF(_L("true")) == 0)))
			iUseBoot2nd = ETrue;
		if (((aValue.CompareF(_L("no")) == 0) || (aValue.CompareF(_L("false")) == 0)))
			iUseBoot2nd = EFalse;
	}
}


void CArloConfig::ParseLine(const TDesC& aLine)
{
//	RDebug::Print(_L("line = %S"), &aLine);

	if (aLine.Length() != 0)
	{
		if (aLine.Left(1) != _L("#"))
		{
			if ((aLine.Left(1) == _L("[")) && aLine.Right(1) == _L("]"))
			{
				NewParsedBootItemL(aLine.Mid(1, aLine.Length() - 2));
			}
			else
			{
				TInt	pos = aLine.LocateF(TChar('='));
			
				if ((pos > 0) && (pos + 1 < aLine.Length()))
				{
					TLex		input(aLine.Left(pos));

					input.SkipSpaceAndMark();
					input.SkipCharacters();
					if (input.TokenLength() != 0)
					{
						TPtrC		token = input.MarkedToken();

						input.Assign(aLine.Mid(pos+1));
						input.SkipSpaceAndMark();
						TPtrC		value = input.Remainder();
						ParseItems(token, value);
					}
				}
			}
		}
	}
}

void CArloConfig::ReadConfigFileL(RFs& aFs, const TDesC& aFileName)
{
	RFile			file;

	User::LeaveIfError(file.Open(aFs, aFileName, EFileRead));

	TInt size;
	User::LeaveIfError(file.Size(size));

	HBufC *buf = HBufC::NewLC(size);
	TPtr	des = buf->Des();
	User::LeaveIfError(file.Read(des, size));

	TBool	finished = EFalse;
	while(!finished)
	{
		TInt pos = des.LocateF(TChar('\n'));
		if (pos >= 0)
		{
			TBool	isDosLine = (des.LocateF(TChar('\r')) >= 0);
			
			ParseLine(des.Left( isDosLine ? pos-1 : pos));
			des = des.Mid(pos + 1);
		}
		else
		{
			ParseLine(des);
			finished = ETrue;
		}
	}

	file.Close();
	CleanupStack::PopAndDestroy();		// buf

	iConfigFile = aFileName;
	if ((iTimeout < -1) || (iTimeout > 30))
	{
		iTimeout = 30;
	}
}


void CArloConfig::Defaults()
{
	iUseBoot2nd = ETrue;
	iTimeout = -1;
	iDefaultBootItem = -1;
	iArloBootItems->Reset();
	iDefaultBootItemString = _L("");
	iConfigFile = _L("<no config file>");

	// set default machine name
	TMachineInfoV1Buf		machineInfo;
	TInt err = UserHal::MachineInfo(machineInfo);
	if (err == KErrNone)
	{
		iMachineName = machineInfo().iMachineName;
	}
	else
	{
		iMachineName = _L("<unknown machine>");
	}
}


void CArloConfig::SetDefaultBootItemByDesciption()
{
	TInt count = iArloBootItems->Count();
	for (TInt i=0; i<count ; i++)
	{
		if (iArloBootItems->At(i).iDescription.CompareF(iDefaultBootItemString) == 0)
		{
			iDefaultBootItem = i;
			break;
		}
	}
	if (iDefaultBootItem == -1)
		iTimeout = -1;
}


void CArloConfig::ConstructL()
{
	const TText**		arloFile;
	const TText*	KArloFileLocations[] =
	{
			// don't change the relative order, we depend on it in read and write routines
		_S("c:\\arlo.cfg"),
		_S("d:\\arlo.cfg"),
		_S("c:\\arlo\\arlo.cfg"),
		_S("d:\\arlo\\arlo.cfg"),
		NULL
	};

	iArloBootItems = new(ELeave) CArrayFixFlat<TArloBootItem>(5);

	RFs		fs;
	User::LeaveIfError(fs.Connect());

	TInt	err;
	Defaults();

	arloFile = &KArloFileLocations[0];
	while(*arloFile)
	{
		TRAP(err, ReadConfigFileL(fs, TPtrC(*arloFile)));
		if (err == KErrNone)
			break;
		
		Defaults();
		arloFile++;
	}
	if (err == KErrNone)
	{
		SetDefaultBootItemByDesciption();
		SanityCheckL();

		if (iConfigFile.Length())
		{
			TFileName	line;
			line.Format(_L("Loaded %S"), &iConfigFile);;
			User::InfoPrint(line);
		}
	}
	else
	{
		SanityCheckL();
	}

	fs.Close();
}


TBool CArloConfig::SwapItems(TInt aIndex1, TInt aIndex2)
{
	TBool	changed = EFalse;
	TInt count = iArloBootItems->Count();
	if ((aIndex1 >= 0) && (aIndex1<count) && (aIndex2 >= 0) && (aIndex2 < count) && (aIndex1 != aIndex2))
	{
		iTempBootItem = iArloBootItems->At(aIndex1);
		iArloBootItems->At(aIndex1) = iArloBootItems->At(aIndex2);
		iArloBootItems->At(aIndex2) = iTempBootItem;
		changed = ETrue;
	}
	return changed;
}


void CArloConfig::DeleteBootItem(TInt aItem)
{
	TInt count = iArloBootItems->Count();
	if ((aItem >= 0) && (aItem<count))
	{
		iArloBootItems->Delete(aItem);
		if (iDefaultBootItem>=aItem)
			iDefaultBootItem--;
	}
}

CArloConfig* CArloConfig::CloneLC()
{
	CArloConfig*	arloConfig = new(ELeave)CArloConfig;
	CleanupStack::PushL(arloConfig);
	arloConfig->iArloBootItems = new(ELeave) CArrayFixFlat<TArloBootItem>(5);
	arloConfig->CloneDataL(this);
	return arloConfig;
}


void CArloConfig::CloneDataL(CArloConfig* aArloConfig)
{
	for (TInt i=0 ; i<aArloConfig->BootItems() ; i++)
		iArloBootItems->AppendL(aArloConfig->BootItem(i));


	iUseBoot2nd = aArloConfig->iUseBoot2nd;
	iTimeout = aArloConfig->iTimeout;
	iDefaultBootItem = aArloConfig->iDefaultBootItem;
	
	iMachineName = aArloConfig->MachineName();
	iConfigFile = aArloConfig->UsedConfigFile();

	iDefaultBootItemString = _L("");
}

TArloBootItem* CArloConfig::CreateNewBootItemLC()
{
	TArloBootItem* arloBootItem = new(ELeave)TArloBootItem;
	CleanupStack::PushL(arloBootItem);
	arloBootItem->iKernelFileName = _L("");
	arloBootItem->iInitrdFileName = _L("");
	arloBootItem->iKernelLoadAdr = 0;
	arloBootItem->iInitrdLoadAdr = 0;
	arloBootItem->iDescription = _L("");
	arloBootItem->iBootString = _L("init=/bin/ash");
	arloBootItem->iForegroundColour = 0;
	arloBootItem->iBackgroundColour = 0;
	arloBootItem->iHasColoursSet = EFalse;
	arloBootItem->iArchNumber = 0;
	return arloBootItem;
}


void CArloConfig::AddBootItemL(TArloBootItem* aArloBootItem)
{
	if (CheckBootItem(*aArloBootItem))
		iArloBootItems->AppendL(*aArloBootItem);
}

TBool CArloConfig::CheckBootItem(const TArloBootItem& aArloBootItem)
{
	if (aArloBootItem.iDescription.Length() == 0)
		return EFalse;
	if (aArloBootItem.iKernelFileName.Length() == 0)
		return EFalse;
	return ETrue;
}


void CArloConfig::SetBootItem(TInt aItem, const TArloBootItem& aArloBootItem)
{
	iArloBootItems->At(aItem) = aArloBootItem;
}



void CArloConfig::SetDefaultBootItem(TInt aDefaultBootItem)
{
	TInt count = iArloBootItems->Count();
	if ((aDefaultBootItem >= -1) && (aDefaultBootItem<count))
	{
		iDefaultBootItem = aDefaultBootItem;
	}
	if (iDefaultBootItem == -1)
		iTimeout = -1;
}


void CArloConfig::SanityCheckL()
{
	if (iArloBootItems->Count() == 0)
	{
		NewParsedBootItemL(EPOC_BOOT_DESCIPTION);
		iDefaultBootItem == -1;
		iTimeout = -1;
	}
}

TBool CArloConfig::HasDescription(const TDesC& aDescription) const
{
	TInt count = iArloBootItems->Count();
	TBool	hasDescription = EFalse;
	for (TInt i=0 ; i<count && !hasDescription ; i++)
	{
		if (iArloBootItems->At(i).iDescription == aDescription)
			hasDescription = ETrue;
	}
	return hasDescription;
}


TBool CArloConfig::BackupOldConfigFile(RFs& aFs, const TDesC& aFileName)
{
	TInfoName	backup = aFileName;

	backup.Replace(backup.Length() - 3, 3, _L("old"));
	aFs.Delete(backup);
	return (aFs.Replace(aFileName, backup) == KErrNone);
}


TBool CArloConfig::Save()
{
	TInt err = KErrNotFound;

	if (iConfigFile.Length() != 0)
	{
		TRAP(err, SaveConfigToFileL(iConfigFile));
	}
	if (err)
	{
//		const TText**	arloFile = NULL;//&KArloFileLocations[1];	// d:\arlo.cfg
		TPtrC  p = _L("d:\\arlo.cfg");
		TRAP(err, SaveConfigToFileL(p));
	}
	if (err)
	{
//		const TText**	arloFile = NULL;//&KArloFileLocations[0];	// c:\arlo.cfg
		TPtrC  p = _L("c:\\arlo.cfg");
		TRAP(err, SaveConfigToFileL(p));
	}

	return (err == KErrNone);
}


void CArloConfig::SaveConfigToFileL(const TDesC& aFileName)
{
	RFs		fsSession;
	User::LeaveIfError(fsSession.Connect());
	CleanupClosePushL(fsSession);

	HBufC*	line = HBufC::NewLC(2 * KMaxBootTextString);

	RFile	file;
	TBool exists = BackupOldConfigFile(fsSession, aFileName);
	if (exists)
		User::LeaveIfError(file.Replace(fsSession, aFileName, EFileWrite));
	else
		User::LeaveIfError(file.Create(fsSession, aFileName, EFileWrite));
	CleanupClosePushL(file);

	line->Des().Format(_L("#\n# This file is automatically generated by Arlo, version %d.%d.%d\n#\n\n"), ARLO_VERSION_MAJOR, ARLO_VERSION_MINOR, ARLO_VERSION_PATCH);
	User::LeaveIfError(file.Write(*line));

	TInt	count = BootItems();
	for (TInt i=0 ; i<count ; i++)
	{
		const TArloBootItem&	bootItem = BootItem(i);

		line->Des().Format(_L("\n[%S]\n"), &bootItem.iDescription);
		User::LeaveIfError(file.Write(*line));

		if (bootItem.iDescription != EPOC_BOOT_DESCIPTION)
		{

			TPtrC	p = KKernelString;
			line->Des().Format(_L("  %S = %S"), &p, &bootItem.iKernelFileName);
			if (bootItem.iKernelLoadAdr)
				line->Des().AppendFormat(_L(" @ 0x%x\n"), bootItem.iKernelLoadAdr);
			else
				line->Des().Append(_L("\n"));
			User::LeaveIfError(file.Write(*line));

			if (bootItem.iInitrdFileName.Length())
			{
				TPtrC	p = KInitRdString;
				line->Des().Format(_L("  %S = %S"), &p, &bootItem.iInitrdFileName);
				if (bootItem.iInitrdLoadAdr)
					line->Des().AppendFormat(_L(" @ 0x%x\n"), bootItem.iInitrdLoadAdr);
				else
					line->Des().Append(_L("\n"));
				User::LeaveIfError(file.Write(*line));
			}

			TPtrC	p1 = KBootString;
			line->Des().Format(_L("  %S = %S\n"), &p1, &bootItem.iBootString);
			User::LeaveIfError(file.Write(*line));

			if (bootItem.iArchNumber)
			{
				TPtrC p = KArchNumber;
				line->Des().Format(_L("  %S = %d\n"), &p, bootItem.iArchNumber);
				User::LeaveIfError(file.Write(*line));
			}
		}
	}

	line->Des().Format(_L("\n\n#\n# General options\n#\n"));
	User::LeaveIfError(file.Write(*line));

	if (iHasUseBoot2nd)
	{
		TPtrC	p2 = KUseBoot2nd;
		TPtrC	value = UseBoot2nd() ? _L("true") : _L("false");
		line->Des().Format(_L("%S = %S\n"), &p2, &value);
		User::LeaveIfError(file.Write(*line));
	}

	if (DefaultBootItem() != -1)
	{
		TPtrC	p = KDefaultBoot;
		line->Des().Format(_L("%S = %S\n"), &p, &BootItem(DefaultBootItem()).iDescription);
		User::LeaveIfError(file.Write(*line));
	}

	if (iTimeout != -1)
	{
		TPtrC	p = KTimeoutString;
		line->Des().Format(_L("%S = %d\n"), &p, iTimeout);
		User::LeaveIfError(file.Write(*line));
	}

	if (iHasMachineName && (iMachineName.Length() > 0 ))
	{
		TPtrC	p = KMachineName;
		line->Des().Format(_L("%S = %S\n"), &p, &iMachineName);
		User::LeaveIfError(file.Write(*line));
	}

	CleanupStack::PopAndDestroy();		// file
	CleanupStack::PopAndDestroy();		// line
	CleanupStack::PopAndDestroy();		// fsSession

	iConfigFile = aFileName;
}

