/*******************************************************************
 *
 *	File:		CArloConfig.h
 *
 *	Author:		Peter van Sebille (peter@yipton.net)
 *
 *	(c) Copyright 2001, Peter van Sebille
 *	All Rights Reserved
 *
 *******************************************************************/

#ifndef __C_ARLO_CONFIG__H
#define __C_ARLO_CONFIG__H

#include "ArloConfigDefs.h"

typedef TBuf<KMaxConfigTextString>	TKernelFileName;
typedef TBuf<KMaxConfigTextString>	TInitrdFileName;
typedef TBuf<KMaxConfigTextString>	TDescription;
typedef TBuf<KMaxBootTextString>	TBootString;

#define EPOC_BOOT_DESCIPTION		_L("EPOC")

class TArloBootItem
{
public:
	TKernelFileName		iKernelFileName;
	TUint				iKernelLoadAdr;
	TInitrdFileName		iInitrdFileName;
	TUint				iInitrdLoadAdr;
	TDescription		iDescription;
	TBootString			iBootString;
	TInt				iArchNumber;
	TUint				iForegroundColour;
	TUint				iBackgroundColour;
	TBool				iHasColoursSet;
};


class CArloConfig : public CBase
{
public:
	~CArloConfig();
	static CArloConfig* ReadConfigFileLC();
	
	TInt BootItems() const {return iArloBootItems->Count();}
	TInt DefaultBootItem() const {return iDefaultBootItem;}
	void SetDefaultBootItem(TInt aDefaultBootItem);
	TInt Timeout() const {return iTimeout;}
	void SetTimeout(TInt aTimeout) {iTimeout = aTimeout;}
	TBool UseBoot2nd() const {return iUseBoot2nd;}
	TBool HasDescription(const TDesC& aDescription) const;

	const TDesC& MachineName() const {return iMachineName;}
	const TDesC& UsedConfigFile() const {return iConfigFile;}

		// edit, update etc
	CArloConfig*			CloneLC();
	const TArloBootItem&	BootItem (TInt aItem) const {return iArloBootItems->At(aItem);}
	void					SetBootItem(TInt aItem, const TArloBootItem&);
	TBool					SwapItems(TInt aIndex1, TInt aIndex2);
	void					DeleteBootItem(TInt aItem);
	TArloBootItem*			CreateNewBootItemLC(); // leaves a TArloBootItem on the Cleanup stack with default parameters
	void					AddBootItemL(TArloBootItem* aArloBootItem);

	TBool Save();

protected:
	CArloConfig();
	void ConstructL();
	void CloneDataL(CArloConfig* aArloConfig);
	void ReadConfigFileL(RFs& aFs, const TDesC& aFileName);
	void ParseLine(const TDesC& aLine);
	void ParseItems(const TDesC& aToken, const TDesC&  aValue);
	void ParseFileNameAndLoadAdr(const TDesC& aString, TDes& aFileName, TUint& aLoadAdr);
	TBool GetNumber(TLex& aLex, TUint& aNumber);
	void NewParsedBootItemL(const TDesC& aBootItem);
	void SetDefaultBootItemByDesciption();
	void SanityCheckL();

	void SaveConfigToFileL(const TDesC& aFileName);
	TBool BackupOldConfigFile(RFs& aFs, const TDesC& aFileName);


	void Defaults();
	TBool CheckBootItem(const TArloBootItem& aArloBootItem);

	TBool			iUseBoot2nd;
	TInt			iTimeout;
	TInt			iDefaultBootItem;

	TDescription	iDefaultBootItemString;	// only used when parsing config file
	TInfoName		iMachineName;
	TFileName		iConfigFile;
	TArloBootItem	iTempBootItem;
	CArrayFixFlat<TArloBootItem>*	iArloBootItems;

	TBool			iHasMachineName;
	TBool			iHasUseBoot2nd;
};


#endif			/* __C_ARLO_CONFIG__H */