/*******************************************************************
 *
 *	File:		ArloApp.cpp
 *
 *	Author:		Peter van Sebille (peter@yipton.net)
 *
 *	(c) Copyright 2001, Peter van Sebille
 *	All Rights Reserved
 *
 *******************************************************************/

#include <e32std.h>
#include <e32cons.h>
#include <f32file.h>
#include <e32svr.h>
#include <w32std.h>

#include "CArloController.h"
#include "CArloEngine.h"

#define DEBUG_PRINTS_PLEASE

#ifdef DEBUG_PRINTS_PLEASE
CConsoleBase*		sConsole = NULL;
#define DBGPRINT(x) sConsole->Printf x
#define DBGPRESSKEY(x) sConsole->Printf x;sConsole->Getch()
#else
#define DBGPRINT(x) /**/
#define DBGPRESSKEY(x) /**/
#endif


#if 0
static void DumpFileUids(CConsoleBase* aConsole, const TDesC& aName, const TDesC& aFileName)
{
	TUint	uid0, uid1, uid2;

	TInt err = GetFileUids(aFileName, uid0, uid1, uid2);
	if (!err)
		aConsole->Printf(_L("%S: uid[0]=0x%x, uid[1]=0x%x, uid[2]=0x%x\n"), &aName, uid0, uid1, uid2);
}


static void DumpUidInfo(CConsoleBase* aConsole)
{
	DumpFileUids(aConsole, _L("Ekern"), EKERN_EXE_FILENAME);
	DumpFileUids(aConsole, _L("EComm"), ECOMM_LDD_FILENAME);
}

#endif


static void ArloMain1L(CConsoleBase* aConsole)
{
	RFs		fsSession;
	User::LeaveIfError(fsSession.Connect());
	CleanupClosePushL(fsSession);

	RWsSession		aWsSession;
#ifndef __WINS__
	User::LeaveIfError(aWsSession.Connect());
	CleanupClosePushL(aWsSession);
#endif

	CArloEngine*	engine = CArloEngine::NewL(&aWsSession, &fsSession);
	CleanupStack::PushL(engine);

	CArloController*	arloController = CArloController::NewLC(*aConsole, engine, fsSession, aWsSession);
	arloController->DoTheArloThingL();

	CleanupStack::PopAndDestroy();		// arloController
	CleanupStack::PopAndDestroy();		// engine

#ifndef __WINS__
	CleanupStack::PopAndDestroy();	// aWsSession;
#endif
	CleanupStack::PopAndDestroy();	// fsSession;
}


void ArloMainL()
{
	CConsoleBase*	console = Console::NewL(_L("ARLO"), TSize(KDefaultConsWidth,KDefaultConsHeight));
	CleanupStack::PushL(console);

#ifdef DEBUG_PRINTS_PLEASE
	sConsole = console;
#endif

	// start by creating a console, so that we can display any errors
	TRAPD(err, ArloMain1L(console));
	if (err)
	{
		console->Printf(_L("Oops, something failed: leave code=%d\n"), err);
		console->Printf(_L("Press a key\n"));
		console->Getch();
	}

	CleanupStack::PopAndDestroy();		// console
}


GLDEF_C TInt E32Main() // main function called by E32
{
	__UHEAP_MARK;

	CTrapCleanup* cleanup=CTrapCleanup::New();

	TRAPD(error, ArloMainL());                     

	__ASSERT_ALWAYS(!error, User::Panic(_L("LINUX"), error));

	delete cleanup; // destroy clean-up stack

	__UHEAP_MARKEND;

	return 0; // and return
}

