/*******************************************************************
 *
 *	File:		ArloShell.cpp
 *
 *	Author:		Peter van Sebille (peter@yipton.net)
 *
 *	(c) Copyright 2001, Peter van Sebille
 *	All Rights Reserved
 *
 *******************************************************************/

#include <e32std.h>
#include <f32file.h>
#include <w32std.h>

static RMutex	gMutex;

static void LoadAndWaitForAppToFinishL(RFs& aFsSession, const TDesC& aApp)
{
	TFindFile	findApprun(aFsSession);
	User::LeaveIfError(findApprun.FindByDir(_L("APPRUN"), _L("\\System\\Programs\\.EXE")));

	RProcess process;
	User::LeaveIfError(process.Create(findApprun.File(), aApp));
	TRequestStatus	status = KRequestPending;
	process.Logon(status);
	process.Resume();
	process.Close();
	User::WaitForRequest(status);
}

static void LoadAndWaitForEpocToFinishL(RFs& aFsSession)
{
	 const TPtrC	app = _L("\\System\\Apps\\Shell\\Shell.APP");
	 LoadAndWaitForAppToFinishL(aFsSession, app);
}

static void LoadAndWaitForArloToFinishL(RFs& aFsSession)
{
	 const TPtrC	app = _L("\\System\\Apps\\Arlo\\Arlo.APP");
	 LoadAndWaitForAppToFinishL(aFsSession, app);
}

static void CheckIfAlreadyRunningL()
{
	TInt err = gMutex.CreateGlobal(_L("ARLOSHELL_MUTEX"));
	if (err)
	{
		User::InfoPrint(_L("ArloShell is already running as a system shell"));
		User::Leave(err);
	}
}

static TBool IsSystemBootL()
{
	RWsSession		wsSession;
	TInt			id=0;
	TInt			count = 0;

	User::LeaveIfError(wsSession.Connect());
	CleanupClosePushL(wsSession);

		/*
		 * We count the number of named window groups.
		 * If we run as the EPOC shell, the count is 0, otherwise there about 10 of them
		 */
	while (id >= 0)
	{
		id = wsSession.FindWindowGroupIdentifier(id , _L("*"));
		if (id > 0)
		{
			TThreadId threadId;
			if (wsSession.GetWindowGroupClientThreadId(id , threadId) == KErrNone)
			{
				RThread	thread;
				if (thread.Open(threadId) == KErrNone)
				{
					thread.Close();
					count++;
//					iConsole.Printf(_L("Threads: thread=%S, id=%d\n"), &name, id);
				}
			}
		}
	}
	CleanupStack::Pop();	// wsSession
	return (count == 0);
}


static void CheckIfSystemShellL()
{
	if (!IsSystemBootL())
	{
		User::InfoPrint(_L("ArloShell can only run as a system shell"));
		User::Leave(KErrGeneral);
	}
}

static void ArloShellL()
{
	RFs		fsSession;
	User::LeaveIfError(fsSession.Connect());
	CleanupClosePushL(fsSession);

	CheckIfAlreadyRunningL();

	CheckIfSystemShellL();

	LoadAndWaitForArloToFinishL(fsSession);

	while(ETrue)
	{
		LoadAndWaitForEpocToFinishL(fsSession);
	}
	
	CleanupStack::PopAndDestroy();	// fsSession;
}



GLDEF_C TInt E32Main() // main function called by E32
{
	__UHEAP_MARK;

	CTrapCleanup* cleanup=CTrapCleanup::New();

	TRAPD(err, ArloShellL()); 

	delete cleanup; // destroy clean-up stack

	__UHEAP_MARKEND;

	return 0; // and return
}

